@@ -77,7 +77,7 @@ namespace tiny_ad {
       return ad(value - x, deriv);
     }
     ad operator* (const double &x) const{
-      return ad(value * x, x * deriv);
+      return ad(value * x, deriv * x);
     }
     ad operator/ (const double &x) const{
       return ad(value / x, deriv / x);
@@ -127,7 +127,7 @@ namespace tiny_ad {
   template<class T, class V>
   ad<T, V> operator/ (const double &x, const ad<T, V> &y) {
     T value = x / y.value;
-    return ad<T, V>(value, (-value / y.value) * y.deriv);
+    return ad<T, V>(value, T(-value / y.value) * y.deriv);
   }
   /* Unary operators with trivial derivatives */
 #define UNARY_MATH_ZERO_DERIV(F)		\
@@ -152,7 +152,7 @@ namespace tiny_ad {
   template<class T, class V>			\
   ad<T, V> F (const ad<T, V> &x){		\
     return ad<T, V>(F (x.value),		\
-		    DF(x.value) * x.deriv);	\
+		    T(DF(x.value)) * x.deriv);	\
   }
   using ::exp;  using ::log;
   using ::sin;  using ::cos;  using ::tan;
@@ -202,7 +202,7 @@ namespace tiny_ad {
   template<class T, class V>
   ad<T, V> pow (const ad<T, V> &x, const double &y){
     return ad<T, V> (pow(x.value, y), // Note: x.value could be 0
-		     y * pow(x.value, y - 1.) * x.deriv);
+		     T( y * pow(x.value, y - 1.) ) * x.deriv);
   }
   /* Comparison operators where a constant is first argument */
 #define COMPARISON_OPERATOR_FLIP(OP1, OP2)			\
@@ -234,7 +234,7 @@ namespace tiny_ad {
   template<int deriv, class T, class V>
   ad<T, V> lgamma (const ad<T, V> &x){
     return ad<T, V> (lgamma< deriv >(x.value),
-		     lgamma< deriv + 1 >(x.value) * x.deriv);
+		     T(lgamma< deriv + 1 >(x.value)) * x.deriv);
   }
   template<class T, class V>
   ad<T, V> lgamma (const ad<T, V> &x){
@@ -259,41 +259,59 @@ namespace tiny_ad {
      Float y = sin(a + b);                  // Run the algorithm
      y.getDeriv();                          // Get all 3rd order derivatives
   */
-#define VARIABLE(order,n) variable<order,n>
-  template<int order, int n>
-  struct variable : ad< VARIABLE(order-1, n),
-			TINY_VECTOR( VARIABLE(order-1, n) , n) > {
-    typedef ad< VARIABLE(order-1, n),
-		TINY_VECTOR(VARIABLE(order-1, n), n) > Base;
-    typedef variable<order-1, n> Type;
-    static const int result_size = n * Type::result_size;
+#define VARIABLE(order, nvar, scalartype) variable<order, nvar, scalartype>
+  template<int order, int nvar, class Double=double>
+  struct variable : ad< VARIABLE(order-1, nvar, Double),
+			TINY_VECTOR( VARIABLE(order-1, nvar, Double) , nvar) > {
+    typedef ad< VARIABLE(order-1, nvar, Double),
+		TINY_VECTOR(VARIABLE(order-1, nvar, Double), nvar) > Base;
+    typedef variable<order-1, nvar, Double> Type;
+    static const int result_size = nvar * Type::result_size;
     variable() { /* Do not zero-initialize */ }
     variable(Base x) : Base(x) {}
     variable(double x) : Base(x) {}
     variable(double x, int id) : Base(x) {
       setid(id);
     }
+    template<class Constant>
+    variable(Constant x) {
+      Base::value = x; Base::deriv.setZero();
+    }
+    template<class Constant>
+    variable(Constant x, int id) {
+      Base::value = x; Base::deriv.setZero();
+      setid(id);
+    }
     void setid(int i0, int count = 0){
       this->value.setid(i0, count);
       this->deriv[i0].setid(i0, count + 1);
     }
-    TINY_VECTOR(double, result_size) getDeriv(){
-      TINY_VECTOR(double, result_size) ans;
-      int stride = result_size / n;
-      for(int i=0; i<n; i++)
+    TINY_VECTOR(Double, result_size) getDeriv(){
+      TINY_VECTOR(Double, result_size) ans;
+      int stride = result_size / nvar;
+      for(int i=0; i<nvar; i++)
 	ans.segment(i * stride, stride) = this->deriv[i].getDeriv();
       return ans;
     }
   };
 #undef VARIABLE
-  template<int n>
-  struct variable<1, n> : ad<double, TINY_VECTOR(double,n) >{
-    typedef ad<double, TINY_VECTOR(double,n) > Base;
-    static const int result_size = n;
-    variable<1, n>() { /* Do not zero-initialize */ }
-    variable<1, n>(Base x) : Base(x) {}
-    variable<1, n>(double x) : Base(x) {}
-    variable<1, n>(double x, int id) : Base(x) {
+  template<int nvar, class Double>
+  struct variable<1, nvar, Double> : ad<Double, TINY_VECTOR(Double,nvar) >{
+    typedef ad<Double, TINY_VECTOR(Double,nvar) > Base;
+    static const int result_size = nvar;
+    variable<1, nvar, Double>() { /* Do not zero-initialize */ }
+    variable<1, nvar, Double>(Base x) : Base(x) {}
+    variable<1, nvar, Double>(double x) : Base(x) {}
+    variable<1, nvar, Double>(double x, int id) : Base(x) {
+      setid(id);
+    }
+    template<class Constant>
+    variable<1, nvar, Double>(Constant x) {
+      Base::value = x; Base::deriv.setZero();
+    }
+    template<class Constant>
+    variable<1, nvar, Double>(Constant x, int id) {
+      Base::value = x; Base::deriv.setZero();
       setid(id);
     }
     void setid(int i0, int count = 0){
@@ -302,7 +320,7 @@ namespace tiny_ad {
       if(count == 1)
 	this->value = 1.0;
     }
-    TINY_VECTOR(double, n) getDeriv(){
+    TINY_VECTOR(Double, nvar) getDeriv(){
       return this->deriv;
     }
   };
